#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <unistd.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "astv.h"

#include "parsecfg.h"

extern FILE *tv;

extern GtkWidget *status;
extern GtkWidget *remote;
extern GtkWidget *remote_small;
extern GtkWidget *settings;
extern GtkWidget *fileselect;
extern GtkWidget *def_cap_entry;

int debug_level = 0;

char cfg_filename[ 1024 ] = "";

char *def_cap_filename = NULL;
char *def_band = NULL;
char *def_size = NULL;
int  def_channel = 0;

cfgStruct cfg[] = {
	/* parameter	type			address of variable */
	{ "capture_filename",	CFG_STRING,		&def_cap_filename },
	{ "default_band",		CFG_STRING,		&def_band },
	{ "default_size",		CFG_STRING,		&def_size },
	{ "default_channel",	CFG_INT,		&def_channel },
	{ NULL, 				CFG_END,		NULL }
};

void read_config() {
	struct stat sbuf;

	if ( cfg_filename[ 0 ] == '\0' )
		sprintf( cfg_filename, "%s/.astvrc", getenv( "HOME" ));

	if ( stat( cfg_filename, &sbuf ))
		write_config();

	cfgParse( cfg_filename, cfg, CFG_SIMPLE );

	if ( debug_level >= 20 ) {
		printf( "%s: filename = '%s'\n", cfg_filename, def_cap_filename );
		printf( "%s: band = '%s'\n", cfg_filename, def_band );
		printf( "%s: size = '%s'\n", cfg_filename, def_size );
		printf( "%s: channel = %d\n", cfg_filename, def_channel );
	}
}

void write_config() {
	FILE *astvrc;
	char tmp_filename[ 1024 ];

	if ( cfg_filename[ 0 ] == '\0' )
		sprintf( cfg_filename, "%s/.astvrc", getenv( "HOME" ));

	if ( ! def_cap_filename ) {
		sprintf( tmp_filename, "%s/astv.pnm", getenv( "HOME" ));
		def_cap_filename = strdup( tmp_filename );

		def_band = strdup( "vhf/uhf" );
		def_size = strdup( "320x240" );
	}

	astvrc = fopen( cfg_filename, "w" );

	if ( astvrc ) {
		fprintf( astvrc, "capture_filename = %s\n", def_cap_filename );
		fprintf( astvrc, "default_band = %s\n", def_band );
		fprintf( astvrc, "default_size = %s\n", def_size );
		fprintf( astvrc, "default_channel = %d\n", def_channel );
		fclose( astvrc );
	}
}

void tv_command( char *cmd ) {
	char msg[1024];

	if ( tv ) {
		fprintf( tv, "%s\n", cmd );
		fflush( tv );

		if ( strstr( cmd, "channel" )) {
			if ( def_channel )
				sprintf( msg, "CH %d", def_channel );
			else
				strcpy( msg, "AUX" );
			set_status( msg );
		}

		/*
		fprintf( stderr, "tv_command( '%s' )\n", cmd );
		fflush( stderr );
		*/
	}
}


void tv_set_defaults() {
	char cmd[ 1024 ];
	char *ptr;

	sprintf( cmd, "resize %s", def_size );
	ptr = strchr( cmd, 'x' );
	if ( ptr )
		*ptr = '\0';
	tv_command( cmd );
	sprintf( cmd, "band %s", def_band );
	tv_command( cmd );
	sprintf( cmd, "channel %d", def_channel );
	tv_command( cmd );
}


void set_status( char *status_msg )
{
	gtk_label_set_text( GTK_LABEL(status), status_msg );
}


void select_ok(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	char cmd[1024];
  
	strcpy( def_cap_filename,
		gtk_file_selection_get_filename(GTK_FILE_SELECTION (fileselect)));
	sprintf( cmd, "capture %s",
		gtk_file_selection_get_filename(GTK_FILE_SELECTION (fileselect)));
	gtk_widget_destroy(GTK_WIDGET(fileselect));
	tv_command( cmd );

	return;
}

void select_cancel(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	gtk_widget_destroy(GTK_WIDGET(fileselect));
	return;
}


void capture_ok(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	strncpy( def_cap_filename,
		gtk_file_selection_get_filename(GTK_FILE_SELECTION (fileselect)), 1023);
	gtk_widget_destroy(GTK_WIDGET(fileselect));

	gtk_entry_set_text( GTK_ENTRY(def_cap_entry), def_cap_filename );

	return;
}

void capture_cancel(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	gtk_widget_destroy(GTK_WIDGET(fileselect));
	return;
}

