#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "astv.h"

GtkWidget *status;
GtkWidget *remote;
GtkWidget *remote_small;
GtkWidget *settings = NULL;
GtkWidget *fileselect;
GtkWidget *def_cap_entry;
GtkWidget *def_ch_entry;


void
power                              (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_main_quit();

	tv_command( "quit" );
	fclose( tv );
	tv = NULL;
	return;
}


void
cb_freeze_image                           (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	tv_command( "freeze" );
}


void
cb_channel_up                             (GtkButton       *button,
                                        gpointer         user_data)
{
	// FIXME
	def_channel++;

	tv_command( "channel_up" );
}


void
cb_channel_down                           (GtkButton       *button,
                                        gpointer         user_data)
{
	// FIXME
	def_channel--;

	tv_command( "channel_down" );
}


void
cb_settings                            (GtkButton       *button,
                                        gpointer         user_data)
{
	if ( ! settings ) {
		settings = create_settings();
		gtk_widget_show( settings );
	} else {
		gtk_widget_hide( settings );
		gtk_widget_destroy( settings );
		settings = NULL;
	}
}

void
cb_capture_image                       (GtkButton       *button,
                                        gpointer         user_data)
{
	if ( ! tv ) return;

	fileselect = create_save_image();
	gtk_widget_show( fileselect );

	gtk_file_selection_set_filename(GTK_FILE_SELECTION (fileselect),
		def_cap_filename );

	gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (fileselect)->cancel_button), "clicked", (GtkSignalFunc) select_cancel, GTK_OBJECT (fileselect));

	gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (fileselect)->ok_button), "clicked", (GtkSignalFunc) select_ok, GTK_OBJECT (fileselect));

	return;
}


gboolean
small_power                                  (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	if ( tv ) {
		tv_command( "quit" );
		fclose( tv );
		tv = NULL;
	} else {
		tv = popen( "/usr/local/bin/xavtv > /dev/null 2> /dev/null", "w" );
		tv_set_defaults();
	}
	return FALSE;
}


void
toggle_enhanced                        (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	if ( remote ) {
		gtk_widget_hide( remote );
		gtk_widget_destroy( remote );
		remote = NULL;
	} else {
		remote = create_remote();
		gtk_widget_show( remote );

		if ( ! tv ) {
			small_power( NULL, NULL, NULL );
		}
	}
}


void
cb_capture_browse_button               (GtkButton       *button,
                                        gpointer         user_data)
{
	fileselect = create_save_image();
	gtk_widget_show( fileselect );

	gtk_file_selection_set_filename(GTK_FILE_SELECTION (fileselect),
		def_cap_filename );

	gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (fileselect)->cancel_button), "clicked", (GtkSignalFunc) capture_cancel, GTK_OBJECT (fileselect));

	gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (fileselect)->ok_button), "clicked", (GtkSignalFunc) capture_ok, GTK_OBJECT (fileselect));

	return;
}


void
cb_save_settings                       (GtkButton       *button,
                                        gpointer         user_data)
{
	write_config();

	gtk_widget_hide( settings );
	gtk_widget_destroy( settings );
	settings = NULL;
}


void
cb_save_cap_entry                      (GtkWidget       *widget,
                                        gpointer         user_data)
{
	def_cap_entry = widget;
	gtk_entry_set_text( GTK_ENTRY(widget), def_cap_filename );
}


void
cb_def_channel_changed                 (GtkEditable     *editable,
                                        gpointer         user_data)
{
	def_channel = atoi( gtk_entry_get_text(GTK_ENTRY(editable)));
}


void
cb_save_ch_entry                       (GtkWidget       *widget,
                                        gpointer         user_data)
{
	char tmp[ 5 ];

	sprintf( tmp, "%d", def_channel );

	def_ch_entry = widget;
	gtk_entry_set_text( GTK_ENTRY(widget), tmp );
}


void
cb_map_status                          (GtkWidget       *widget,
                                        gpointer         user_data)
{
	status = widget;
}


void
cb_def_filename_changed                (GtkEditable     *editable,
                                        gpointer         user_data)
{
	free( def_cap_filename );
	def_cap_filename = strdup( gtk_entry_get_text(GTK_ENTRY(editable)));
}


void
cb_mute                                (GtkButton       *button,
                                        gpointer         user_data)
{
	tv_command( "mute" );
}


void
cb_map_settings                        (GtkWidget       *widget,
                                        gpointer         user_data)
{
	GtkWidget *toggle = NULL;
	char name[ 20 ];

	if ( ! strcmp( def_band, "vhf/uhf" )) {
		toggle = lookup_widget( widget, "radio_vhfuhf" );
	} else if ( ! strcmp( def_band, "catv1" )) {
		toggle = lookup_widget( widget, "radio_catv1" );
	} else if ( ! strcmp( def_band, "catv2" )) {
		toggle = lookup_widget( widget, "radio_catv2" );
	}

	if ( toggle ) {
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(toggle), 1 );
	}

	strcpy( name, "rb" );
	strcat( name, def_size );
	toggle = lookup_widget( widget, name );
	if ( ! toggle ) {
		fprintf( stderr, "ERROR: invalid default size, setting to 320x240\n" );
		toggle = lookup_widget( widget, "rb320x240" );
	}

	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(toggle), 1 );
}


void
cb_band_changed                        (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *vhfuhf;
	GtkWidget *catv1;
	GtkWidget *catv2;
	char tmp[ 20 ];

	if ( def_band ) {
		free( def_band );
		def_band = strdup( "vhf/uhf" );
	}

	vhfuhf = lookup_widget( GTK_WIDGET(button), "radio_vhfuhf" );
	catv1 = lookup_widget( GTK_WIDGET(button), "radio_catv1" );
	catv2 = lookup_widget( GTK_WIDGET(button), "radio_catv2" );

	if ( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(vhfuhf))) {
		free( def_band );
		def_band = strdup( "vhf/uhf" );
	} else if ( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(catv1))) {
		free( def_band );
		def_band = strdup( "catv1" );
	} else if ( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(catv2))) {
		free( def_band );
		def_band = strdup( "catv2" );
	}

	sprintf( tmp, "band %s", def_band );
	tv_command( tmp );
}


void
cb_size_changed                        (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *s320x240;
	GtkWidget *s280x210;
	GtkWidget *s240x180;
	GtkWidget *s200x150;
	GtkWidget *s160x120;
	char size[ 20 ];
	char *ptr;

	if ( def_size ) {
		free( def_size );
		def_size = strdup( "320x240" );
	}

	s320x240 = lookup_widget( GTK_WIDGET(button), "rb320x240" );
	s280x210 = lookup_widget( GTK_WIDGET(button), "rb280x210" );
	s240x180 = lookup_widget( GTK_WIDGET(button), "rb240x180" );
	s200x150 = lookup_widget( GTK_WIDGET(button), "rb200x150" );
	s160x120 = lookup_widget( GTK_WIDGET(button), "rb160x120" );

	if ( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(s320x240))) {
		free( def_size );
		def_size = strdup( "320x240" );
	} else if ( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(s280x210))) {
		free( def_size );
		def_size = strdup( "280x210" );
	} else if ( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(s240x180))) {
		free( def_size );
		def_size = strdup( "240x180" );
	} else if ( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(s200x150))) {
		free( def_size );
		def_size = strdup( "200x150" );
	} else if ( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(s160x120))) {
		free( def_size );
		def_size = strdup( "160x120" );
	}

	sprintf( size, "resize %s", def_size );
	ptr = strchr( size, 'x' );
	if ( ptr )
		*ptr = '\0';
	tv_command( size );
}
